==> oscars_mgmnt_clms-0.0.5.dev20240208+28-py3-none-any.whl: oscars_mgmnt_clms/management.py <==
"""
Created on 02 Jully 2019

@author: Tonneau Stephane
"""

import os
import json
import re
from oscars_mgmnt_clms.utils.es_support import ElasticSupport
from oscars_mgmnt_clms.utils.common import ResourceReader, Jsonify, JsonValidator

resource_path = os.path.normpath(os.path.join(os.path.split(__file__)[0], "resources"))


class ElasticsearchObject(object):
    '''class that deals with elasticsearch entries'''

    def __init__(self, config, index):
        """
        Initialisator of the class
        """
        self.es_collection_index = index
        self.es_connector = ElasticSupport(config)

    def get_string(self, uid):
        """ gets the entry referenced by the uid as a string
        :param uid: the urn reference
        :type uid: a string

        :returns: the json string that describe entry.
        :rtype: a string
        """
        res = self.get(uid)
        if res is not None:
            res = json.dumps(res, indent=4)
        return res

    def get(self, uid):
        """ gets the  entry referenced by the uid as json
        :param uid: the urn reference
        :type uid: a string

        :returns: the json structure that describe the entry.
        :rtype: a json structure
        """
        res = None
        try:
            res = self.es_connector.get(self.es_collection_index, uid)['_source']
        finally:
            return res

    def get_json(self, filename):
        """ gets the json representation of the content of a filename
        :param filename: the filename that contains json structure.
        :type filename: a string

        :returns: the json representation of the filename.
        :rtype: a json structure
        """
        with open(filename, 'r') as file:
            data = file.read()
        return json.loads(data)

    def put_file(self, filename, schema=None, modifier=None):
        """ puts or updates a given entry
        :param filename: the filename that contains json structure.
        :type filename: a string

        :returns: the result of the elasticsearch insert
        :rtype: a status
        """
        data = self.get_json(filename)
        if modifier is not None:
            data = modifier(data)
        return self.put(data, schema)

    def put(self, data, schema=None, _id=None):
        """ puts or updates a given entry
        :param data: the json structure that contains json structure.
        :type data: a json structure

        :returns: the result of the elasticsearch insert
        :rtype: a status
        """
        if schema is not None:
            json_verif = Jsonify(None, JsonValidator(schema))
            json_verif.content = data
            json_verif.to_geoJson()

        id = _id if _id else data["id"]
        res = self.es_connector.single_insert(self.es_collection_index, data, id)
        return res

    def delete(self, uid):
        """ deletes an entry
        :param uid: the urn reference of the entry
        :type uid: a string

        :returns: the result of the deletion
        :rtype: a string
        """
        hits = self.es_connector.search(self.es_collection_index,
                                        ElasticSupport.get_search_coll(uid))
        hits = hits['hits']['hits']
        res = None
        if len(hits) > 0:
            # the entry was there
            res = self.es_connector.es_connector.delete(self.es_collection_index, hits[0]['_id'])
        return res

    def listIDs(self, wildcard):
        generator = self.es_connector.search_all(self.es_collection_index, ElasticSupport.get_search_wildcard("id", wildcard))
        ids = [doc['_source']['id'] for doc in generator]
        return ids


class CollectionDefinitions(ElasticsearchObject):
    """class that deals with collection definitions entries"""

    def __init__(self, config, index=None):
        """
        Initialisator of the class
        """
        the_index = config.collections_index if index is None else index
        ElasticsearchObject.__init__(self, config, the_index)

    def list(self):
        """ returns the full list of existing collections

        :returns: a list of existing collections, with their urn as key
        :rtype: a dictionary
        """
        hits = self.es_connector.search(self.es_collection_index,
                                        ElasticSupport.get_search_all())
        hits = hits['hits']['hits']
        res = {}
        for hit in hits:
            res[hit['_source']['id']] = hit['_source']['hiddenInformation']['esIndex']
        return res


class CollectionMgmnt(object):
    """class that deals with product metadata storage"""

    def __init__(self, config, es_index=None):
        """
        Initialisator of the class
        """
        self.es_connector = ElasticSupport(config)
        self.es_collection_index = config.collections_index if es_index is None else es_index

    def _create(self, es_index: str, mappings: str, alias: bool = True):
        """ Creates a new index for a specific document type, with the provided mapping template.
        Appends a number to the index name and creates an alias for it.

        :param es_index: the index to create
        :param mappings: filename of the mapping definition to use
        :param alias: boolean indicating whether to move or create the alias

        :returns: the result of the creation
        :rtype: a dictionary
        """
        data = ResourceReader.get_reader(mappings).get_string()
        data = data.replace('\n', '').replace('\r', '')
        # data = data.replace("{{estype}}", es_type)

        # append number to index
        old_indices = self.es_connector.es_connector.indices.get(es_index + "*")
        pattern = re.compile(es_index + r"[_-](\d*)$")
        append_number = max([int(pattern.search(ci).group(1)) for ci in old_indices if pattern.match(ci)], default=0) + 1
        new_index = es_index + '_{:04d}'.format(append_number)

        created_index = self.es_connector.es_connector.indices.create(new_index, data)

        # create alias for newly created index if alias is requested and no index exists with the same name
        if alias and es_index not in old_indices:
            self.alias(es_index, new_index)

        return created_index

    def initialise(self, es_index=None):
        """ creates a collection definition ES index, based on the configuration
        file

        :returns: the result of the creation
        :rtype: a dictionary
        """
        collection_index = self.es_collection_index if es_index is None else es_index
        return self._create(collection_index, os.path.join(resource_path, "mappings/collection_mapping.def"))


    def drop(self):
        """ deletes the collection definition

        :returns: the result of the deletion
        :rtype: a dictionary
        """
        # Delete an index
        return self.es_connector.es_connector.indices.delete(self.es_collection_index)

    def create(self, es_index: str, alias: bool):
        """ creates a product collection, on the provided index
        :param es_index: the index of the product collection to create
        :param alias: boolean indicating whether to move or create the alias

        :returns: the result of the creation
        :rtype: a dictionary
        """
        return self._create(es_index, os.path.join(resource_path, "mappings/product_mapping.def"), alias=alias)

    def remove(self, es_index):
        """ removes a product collection, on the provided index
        :param es_index: the index of the product collection to delete
        :type es_index: a string

        :returns: the result of the deletion
        :rtype: a dictionary
        """
        # Delete an index
        # if this is an alias, remove it and the corresponding index
        if self.es_connector.es_connector.indices.exists_alias(name=es_index):
            alias_info = self.es_connector.es_connector.indices.get_alias(name=es_index)
            print("Deleting alias {}".format(es_index))
            self.es_connector.es_connector.indices.delete_alias(index="*", name=es_index)
            for index in alias_info:
                print("Deleting index {}".format(index))
                self.es_connector.es_connector.indices.delete(index)
            return

        else:
            return self.es_connector.es_connector.indices.delete(es_index)

    def alias(self, alias: str, index: str):
        """
        Set or move the Elasticsearch alias to the specified index.

        :param alias: Elasticsearch alias
        :param index: Elasticsearch index
        """
        # if alias exists, remove it
        if self.es_connector.es_connector.indices.exists_alias(name=alias):
            self.es_connector.es_connector.indices.delete_alias(index="*", name=alias)
        # put alias
        self.es_connector.es_connector.indices.put_alias(index, alias)

==> oscars_mgmnt_clms-0.1.5.dev20240611+73-py3-none-any.whl: oscars_mgmnt_clms/management.py <==
"""
Created on 02 Jully 2019

@author: Tonneau Stephane
"""

import os
import json
import re
from oscars_mgmnt_clms.utils.es_support import ElasticSupport
from oscars_mgmnt_clms.utils.common import ResourceReader, Jsonify, JsonValidator

resource_path = os.path.normpath(os.path.join(os.path.split(__file__)[0], "resources"))


class ElasticsearchObject(object):
    """class that deals with elasticsearch entries"""

    def __init__(self, config, index):
        """
        Initialisator of the class
        """
        self.es_collection_index = index
        self.es_connector = ElasticSupport(config)

    def get_string(self, uid):
        """gets the entry referenced by the uid as a string
        :param uid: the urn reference
        :type uid: a string

        :returns: the json string that describe entry.
        :rtype: a string
        """
        res = self.get(uid)
        if res is not None:
            res = json.dumps(res, indent=4)
        return res

    def get(self, uid):
        """gets the  entry referenced by the uid as json
        :param uid: the urn reference
        :type uid: a string

        :returns: the json structure that describe the entry.
        :rtype: a json structure
        """
        res = None
        try:
            res = self.es_connector.get(self.es_collection_index, uid)["_source"]
        finally:
            return res

    def get_json(self, filename):
        """gets the json representation of the content of a filename
        :param filename: the filename that contains json structure.
        :type filename: a string

        :returns: the json representation of the filename.
        :rtype: a json structure
        """
        with open(filename, "r") as file:
            data = file.read()
        return json.loads(data)

    def put_file(self, filename, schema=None, modifier=None):
        """puts or updates a given entry
        :param filename: the filename that contains json structure.
        :type filename: a string

        :returns: the result of the elasticsearch insert
        :rtype: a status
        """
        data = self.get_json(filename)
        if modifier is not None:
            data = modifier(data)
        return self.put(data, schema)

    def put(self, data, schema=None, _id=None):
        """puts or updates a given entry
        :param data: the json structure that contains json structure.
        :type data: a json structure

        :returns: the result of the elasticsearch insert
        :rtype: a status
        """
        if schema is not None:
            json_verif = Jsonify(None, JsonValidator(schema))
            json_verif.content = data
            json_verif.to_geoJson()

        id = _id if _id else data["id"]
        res = self.es_connector.single_insert(self.es_collection_index, data, id)
        return res

    def delete(self, uid):
        """deletes an entry
        :param uid: the urn reference of the entry
        :type uid: a string

        :returns: the result of the deletion
        :rtype: a string
        """
        hits = self.es_connector.search(
            self.es_collection_index, ElasticSupport.get_search_coll(uid)
        )
        hits = hits["hits"]["hits"]
        res = None
        if len(hits) > 0:
            # the entry was there
            res = self.es_connector.es_connector.delete(
                self.es_collection_index, hits[0]["_id"]
            )
        return res

    def listIDs(self, wildcard):
        generator = self.es_connector.search_all(
            self.es_collection_index, ElasticSupport.get_search_wildcard("id", wildcard)
        )
        ids = [doc["_source"]["id"] for doc in generator]
        return ids


class CollectionDefinitions(ElasticsearchObject):
    """class that deals with collection definitions entries"""

    def __init__(self, config, index=None):
        """
        Initialisator of the class
        """
        the_index = config.collections_index if index is None else index
        ElasticsearchObject.__init__(self, config, the_index)

    def list(self):
        """returns the full list of existing collections

        :returns: a list of existing collections, with their urn as key
        :rtype: a dictionary
        """
        hits = self.es_connector.search(
            self.es_collection_index, ElasticSupport.get_search_all()
        )
        hits = hits["hits"]["hits"]
        res = {}
        for hit in hits:
            res[hit["_source"]["id"]] = hit["_source"]["hiddenInformation"]["esIndex"]
        return res


class CollectionMgmnt(object):
    """class that deals with product metadata storage"""

    def __init__(self, config, es_index=None):
        """
        Initialisator of the class
        """
        self.es_connector = ElasticSupport(config)
        self.es_collection_index = (
            config.collections_index if es_index is None else es_index
        )

    def _create(self, es_index: str, mappings: str, alias: bool = True):
        """Creates a new index for a specific document type, with the provided mapping template.
        Appends a number to the index name and creates an alias for it.

        :param es_index: the index to create
        :param mappings: filename of the mapping definition to use
        :param alias: boolean indicating whether to move or create the alias

        :returns: the result of the creation
        :rtype: a dictionary
        """
        data = ResourceReader.get_reader(mappings).get_string()
        data = data.replace("\n", "").replace("\r", "")
        # data = data.replace("{{estype}}", es_type)

        # append number to index
        old_indices = self.es_connector.es_connector.indices.get(es_index + "*")
        pattern = re.compile(es_index + r"[_-](\d*)$")
        append_number = (
            max(
                [
                    int(pattern.search(ci).group(1))
                    for ci in old_indices
                    if pattern.match(ci)
                ],
                default=0,
            )
            + 1
        )
        new_index = es_index + "_{:04d}".format(append_number)

        created_index = self.es_connector.es_connector.indices.create(new_index, data)

        # create alias for newly created index if alias is requested and no index exists with the same name
        if alias and es_index not in old_indices:
            self.alias(es_index, new_index)

        return created_index

    def initialise(self, es_index=None):
        """creates a collection definition ES index, based on the configuration
        file

        :returns: the result of the creation
        :rtype: a dictionary
        """
        collection_index = self.es_collection_index if es_index is None else es_index
        return self._create(
            collection_index,
            os.path.join(resource_path, "mappings/collection_mapping.def"),
        )

    def drop(self):
        """deletes the collection definition

        :returns: the result of the deletion
        :rtype: a dictionary
        """
        # Delete an index
        return self.es_connector.es_connector.indices.delete(self.es_collection_index)

    def create(self, es_index: str, alias: bool):
        """creates a product collection, on the provided index
        :param es_index: the index of the product collection to create
        :param alias: boolean indicating whether to move or create the alias

        :returns: the result of the creation
        :rtype: a dictionary
        """
        return self._create(
            es_index,
            os.path.join(resource_path, "mappings/product_mapping.def"),
            alias=alias,
        )

    def remove(self, es_index):
        """removes a product collection, on the provided index
        :param es_index: the index of the product collection to delete
        :type es_index: a string

        :returns: the result of the deletion
        :rtype: a dictionary
        """
        # Delete an index
        # if this is an alias, remove it and the corresponding index
        if self.es_connector.es_connector.indices.exists_alias(name=es_index):
            alias_info = self.es_connector.es_connector.indices.get_alias(name=es_index)
            print("Deleting alias {}".format(es_index))
            self.es_connector.es_connector.indices.delete_alias(
                index="*", name=es_index
            )
            for index in alias_info:
                print("Deleting index {}".format(index))
                self.es_connector.es_connector.indices.delete(index)
            return

        else:
            return self.es_connector.es_connector.indices.delete(es_index)

    def alias(self, alias: str, index: str):
        """
        Set or move the Elasticsearch alias to the specified index.

        :param alias: Elasticsearch alias
        :param index: Elasticsearch index
        """
        # if alias exists, remove it
        if self.es_connector.es_connector.indices.exists_alias(name=alias):
            self.es_connector.es_connector.indices.delete_alias(index="*", name=alias)
        # put alias
        self.es_connector.es_connector.indices.put_alias(index, alias)
